// Polling Place Capacity Simulation written by Mark Pelczarski, mail@pelczarski.name
// Based on research by Carrol Chang, and informed with previous research by
// Douglas M. Spencer and Zachary S. Markovits, "Long Lines at Polling Stations? Observations from an Election Day Field Study"
// William A. Edelstein and Arthur D. Edelstein, "Queuing and Elections: Long Lines, DREs and Paper Ballots"
// Elora Mukherjee, "Abolishing the Time Tax on Voting"
// Marist College Institute for Public Opinion, http://maristpoll.marist.edu

// Javascript support libraries required: JQuery and ElyCharts (elycharts.com), which in turn requires Raphael JS.


// These are the default values on startup of the simulation
var DEFAULTTURNOUT = 500;
var DEFAULTREGISTER = 80;
var VOTERSPERCHECKSTATION = 400;
var VOTERSPERVOTESTATION = 110;
var VOTERSPERREGSTATION = 70;
var IDCHECKINMINUTES = '1.75';

// These are the walk-off percentages used from the Marist polls.
// up to 30 min 27%, 31-60 min 17%, 60+ 15%, "as long as it takes" 41%
// array 0 is < 30 min, 1 is 30-60, 2 is 60-90, 3 is > 90
// under 30, 0% walk off. 30-60 27% walk off. 60-90 27+17=44% walk. 90: 27+17+15=59% walk
var WALKOFFPERCENTS = new Array(0,0.27,0.44,0.59); 

// More defaults, but these can be changed (when working with uploaded data and these values not included in upload,
// changing the corresponding textboxes changes the default used for the missing values
var DefaultCheckinMinutes = '1.5';
var DefaultVoteMinutes = '5';
var DefaultRegisterMinutes = '8';
var DefaultScanMinutes = '1';

var DefaultEarlyArrivals = '1.2'; 
var DefaultCannotCheckIn = '2.0';

// Global parameters
var debug = false;  // true turns on debug mode; for development purposes only
var samedayreg = false; // set true when same day registration option is checked
var unsorted = true; // keeps track of whether alldata is sorted by county/precinct name
// Clicking table headings can re-sort the data by results instead

// This JQuery function is executed on initial page load for initialization
$(function(){

	// Add ?debug to URL to turn on extra debugging output; for development purposes only
	if (window.location.search.indexOf('debug')>=0) debug = true;
	
	// Set all the default form parameters
	$('#checkinminutes').val(DefaultCheckinMinutes);
	$('#voteminutes').val(DefaultVoteMinutes);
	$('#regminutes').val(DefaultRegisterMinutes);
	$('#scanminutes').val(DefaultScanMinutes);
	$('#voters').val(DEFAULTTURNOUT);
	$('#register').val(DEFAULTREGISTER);
	$('#checkinstations').val(Math.ceil(DEFAULTTURNOUT/VOTERSPERCHECKSTATION));
	$('#votingstations').val(Math.ceil(DEFAULTTURNOUT/VOTERSPERVOTESTATION));
	$('#scanners').val(1);
	$('#nocheckin').val(DefaultCannotCheckIn);
	
	// This .js file may be uploaded 5/2016 to VTP site before the HTML file on that site is edited. Once edited,
	// the textbox 'early' with exist, but until then check its existence first. Once the HTML file is updated,
	// can search for all instances of "if ($('#early').length > 0)" and remove the condition.
	if ($('#early').length > 0) $('#early').val(DefaultEarlyArrivals);
	
	// This is legacy code from when county and precinct data could be loaded via a JSON file in the
	// simulation directory. This has not been used nor updated in recent versions of the simulation,
	// so this condition should always evaluate as false. Previous code for this feature has been
	// left intact in case there is ever a need to resurrect it.
	// "data.js" was "included" in the HTML file. Did it exist??
	if (typeof alldata !== 'undefined')
		initData();
	else
		// initial run of the simulation and charting of the results
		drawChart(sim(0,0));

	// Program the actions of each of the form controls using JQuery notation
	$('#waitbutton').click(function() { simAllCounty(); }); // run the simulation on all precincts, no charting
	$('#downloadbutton').click(function(e) {e.preventDefault(); downloadData(); });
	$('#uploadbutton').click(function(e)	{e.preventDefault(); showUploadDialog();}); 
	$('#uploadokbutton').click(function() {uploadData();}); 
	$('#uploadcancelbutton').click(function() {closeUploadDialog();} );
	
	// Changes to data values will trigger re-running the simulation
	$('#jsbutton').click(function() { drawChartSim(this); });
	$('#voters').change(function() { if (validInt('#voters')) drawChartSim(this); });
	$('#register').change(function() { if (validInt('#register')) drawChartSim(this); });
	$('#checkinstations').change(function() { drawChartSim(this); });
	$('#votingstations').change(function() { drawChartSim(this); });
	$('#regstations').change(function() { drawChartSim(this); });
	$('#scanners').change(function() { drawChartSim(this); });
	$('#checkinminutes').change(function() { 
		if (validFloat('#checkinminutes'))
		{
			// if working with uploaded data, this now value will replace any missing corresponding values
			if (typeof alldata !== 'undefined') DefaultCheckinMinutes=$('#checkinminutes').val();
			drawChartSim(this);
		}
	});
	$('#regminutes').change(function() { 
		if (validFloat('#regminutes'))
		{
			// if working with uploaded data, this now value will replace any missing corresponding values
			if (typeof alldata !== 'undefined') DefaultRegisterMinutes=$('#regminutes').val();
			drawChartSim(this);
		}
	});
	$('#voteminutes').change(function() { 
		if (validFloat('#voteminutes')) 
		{
			// if working with uploaded data, this now value will replace any missing corresponding values
			if (typeof alldata !== 'undefined') DefaultVoteMinutes=$('#voteminutes').val();
			drawChartSim(this);
		}
	});
	$('#scanminutes').change(function() { 
		if (validFloat('#scanminutes')) 
		{
			// if working with uploaded data, this now value will replace any missing corresponding values
			if (typeof alldata !== 'undefined') DefaultScanMinutes=$('#scanminutes').val();
			drawChartSim(this);
		}
	});
	$('#nocheckin').change(function() { 
		if (validFloat('#nocheckin')) 
		{
			// if working with uploaded data, this now value will replace any missing corresponding values
			if (typeof alldata !== 'undefined') DefaultCannotCheckIn=$('#nocheckin').val();
			drawChartSim(this);
		}
	});	
	
	if ($('#early').length > 0) $('#early').change(function() {
		if (validFloat('#early')) 
		{
			// if working with uploaded data, this now value will replace any missing corresponding values
			if (typeof alldata !== 'undefined') DefaultEarlyArrivals=$('#early').val();
			drawChartSim(this);
		}
	});	
	
	$('#arrivals').change(function() { drawChartSim(this); });
	$('#opentime').change(function() { drawChartSim(this); });
	$('#closetime').change(function() { drawChartSim(this); });
	
	// Clicking these options will redraw the chart with/without optional curves shown
	$('#showcurve').click(function() { drawChart(lastsim); });
	$('#showactual').click(function() { drawChart(lastsim); });
	$('#showrange').click(function() { drawChart(lastsim); });

	// If the same-day registration or scanner checkboxes are clicked, show or hide the
	// extra data options that are required
	$('#samedayregistration').click(function() 
		{	if ($('#samedayregistration').is(":checked"))
			{ samedayreg=true; $('.samedayreg').css('display','inline');}
			else
			{ samedayreg=false; $('.samedayreg').css('display','none');}
			drawChartSim(this);
		});

	$('#scannerlines').click(function()
		{	if ($('#scannerlines').is(":checked"))
				$('.scanners').css('display','inline');
			else
				$('.scanners').css('display','none');
			drawChartSim(this);
		});

	// Set this so the enter key in a text box fires the change event above and doesn't trigger the upload button:
	$(document).keypress(function(e){if (e.which==13) {e.preventDefault();$('#jsbutton').focus();}}); 
	
	// JQuery modified accordian display, enables the +/- toggles for showing advanced features.
	// Set a click on any 'h' tag an option div to toggle the 
	// visibility of the div immediately after the 'h' tag.
	$('.option :header').click(function() {
		// toggle visibility of the div under the option name
		$(this).next().toggle('fast');
		// toggle the +/- sign in front of the option name
		var symbol=$(this).find('span');
		symbol.html((symbol.html()=='+')? '-':'+');
		return false;
	}).next().hide();
});
// End of initialization code


//*************************************************************************************************************
// IGNORE THE FOLLOWING FUNCTION
// The initData function that follows is legacy code that was originally used for loading precinct and county
// data via JSON. This feature is no longer used, but the code has been left intact here in case it ever
// needs to be re-implemented.
// This executes if 'data.js' was successfully loaded with JSON 'alldata' instance.
// THIS HAS NOT BEEN UPDATED WITH SCANNERS OR CHECKBOXES FOR SAME-DAY REG AND SCANNERS
// ANY UPDATE WILL HAVE TO INCLUDE ADDITION OF alldata.columns
function initData()
{
	$('.data').css('display','inline');
	/*
	CHANGE THIS TO "IF alldata.precinctdata[i].registervoters is defined": (remove .samedayreg? Only use is here.)
	if (alldata.samedayreg==1)
	{
		samedayreg=true; // NEED TO ADDRESS THE ADDED CHECKBOX NOW. THIS NEEDS MODIFICATION
		  // IF WE RE-IMPLEMENT THE JSON INPUT
		$('.samedayreg').css('display','inline');
	}
	*/
	// If ID is required for voting, change check-in average to IDCHECKINMINUTES minutes
	if (alldata.voterid==1) $('#checkinminutes').val(IDCHECKINMINUTES);

	// initialize values for check-in, voting, and registration stations
	for (var i=0; i<alldata.precinctdata.length; i++)
	{
		var totalvoters=alldata.precinctdata[i].turnout;
		if (samedayreg) 
		{
			var regvoters=alldata.precinctdata[i].registervoters;
			alldata.precinctdata[i].regstations = Math.ceil(regvoters/VOTERSPERREGSTATION);
			alldata.precinctdata[i].checkinstations = Math.ceil((totalvoters-regvoters)/VOTERSPERCHECKSTATION);
		}
		else
		{
			alldata.precinctdata[i].checkinstations = Math.ceil(totalvoters/VOTERSPERCHECKSTATION);
			alldata.precinctdata[i].regstations = 1; // ADDED A DEFAULT REG STATION, IN CASE BOX IS CHECKED BY USER
		}
		alldata.precinctdata[i].scanstations = 0; // DEFAULT IS NO SCANNERS/SCAN LINES
		alldata.precinctdata[i].votingstations = Math.ceil(totalvoters/VOTERSPERVOTESTATION);
		alldata.precinctdata[i].maxwait = -99999; // max wait
		alldata.precinctdata[i].avgwait = -99999; // avg wait
		alldata.precinctdata[i].walkoffs = -99999; // walk offs
	}

	showCounties();
	getPrecincts(); // calls drawChart
}
// THE ABOVE IS UNUSED IN THE CURRENT VERSION. CONTINUE READING ACTIVE CODE BELOW
//*************************************************************************************************************

// Create the county/city option list
function showCounties()
{
	// sort by county/city then precinct
	alldata.precinctdata.sort(precinctCompare);
	unsorted = false;
	var html='';
	var c='';
	for (var i=0;i<alldata.precinctdata.length;i++)
	{
		if (alldata.precinctdata[i].county != c)
		{
			c = alldata.precinctdata[i].county;
			html+='<option>'+c+'</option>';
		}
	}	
	$('#county').html(html);
}

// Compare precincts for sorting, alphabetically by county/precinct name
function precinctCompare(a,b)
{
	// first compare by county
	if (a.county < b.county) return -1;
	if (a.county > b.county) return 1;
	// if county names are the same, compare precinct names
	if (a.precinct < b.precinct) return -1;
	if (a.precinct > b.precinct) return 1;
	return 0;
}

// Compare precincts for sorting ascending by id (used in download function)
function idCompare(a,b)
{
	if (parseInt(a.id) < parseInt(b.id)) return -1;
	if (parseInt(a.id) > parseInt(b.id)) return 1;
	return 0;
}

// Populate the precinct option list and the table
function getPrecincts()
{
	// sort by county/city then precinct
	if (unsorted) {alldata.precinctdata.sort(precinctCompare); unsorted=false;}
	var c = $('#county').val();
	var html='';
	// cycle through all the data
	for (var i=0;i<alldata.precinctdata.length;i++)
	{
		// and if the county matches the current county, add this precinct to the option list
		if (alldata.precinctdata[i].county == c)
			html+='<option>'+alldata.precinctdata[i].precinct+'</option>';
	}
	$('#precinct').html(html);
	// then generate the table too
	fillTable();
	// and populate the currently displayed data with the first precinct in the list
	getPrecinctData($("#county"));
}

// Populate the precinct edit table with the data for the currently selected county
function fillTable()
{
	var c = $('#county').val();
	var table='<tr><th class="click" onclick="sortPrecinct();">Precinct</th><th>Check-in</th><th>Voting</th>';
	// Optional columns conditions: samedayreg tells whether need registration stations column
	// if alldata.columns.ss < 99 then need scanners column
	// for the scanners data, if scanners = 0 then enable=false for the textbox
	if (samedayreg) table += '<th>Registration</th>';
	if (alldata.columns.ss < 99) table += '<th>Scanners</th>';
	table += '<th class="click" onclick="sortWait();">Peak Wait'
		+'<span class="tip"><img src="capacityfiles/question12x12.png">' 
		+'<span>Average wait for voters arriving<br>during the worst 15-minute interval<br>throughout the day.</span></span></th>'
		+'<th class="click" onclick="sortAvgWait();">Avg. Wait'
		+'<span class="tip"><img src="capacityfiles/question12x12.png">' 
		+'<span>Average wait among all voters<br>throughout the day.</span></span></th>'
		+'<th class="click" onclick="sortWalk();">Walkoff</th>';
	table+='</tr>';

	// cycle through all the data
	for (var i=0;i<alldata.precinctdata.length;i++)
	{
		// and if the county matches the current county, add this one to the table
		if (alldata.precinctdata[i].county == c)
		{
			var mw = alldata.precinctdata[i].maxwait;
			if (mw == -99999) mw='';
			var aw = alldata.precinctdata[i].avgwait;
			if (aw == -99999) aw='';
			var wo = alldata.precinctdata[i].walkoffs;
			if (wo == -99999) wo='';

			table+='<tr><td>'+alldata.precinctdata[i].precinct+'</td><td><input type="text" id="cs'+alldata.precinctdata[i].id
				+'" value="'+alldata.precinctdata[i].checkinstations
				+'" size=4 onchange="textc(this);"></td><td><input type="text" id="vs'+alldata.precinctdata[i].id
				+'" value="'+alldata.precinctdata[i].votingstations+'" size=4 onchange="textc(this);"></td>';
			
			// optional registration stations column
			if (samedayreg) table += '<td><input type="text" id="rs'+alldata.precinctdata[i].id
				+'" value="'+alldata.precinctdata[i].regstations+'" size=4 onchange="textc(this);"></td>';
			
			// optional scanners column
			if (alldata.columns.ss < 99)
			{	
				var enable = (alldata.precinctdata[i].scanstations==0)?'disabled':'onchange="textc(this);"';
				table += '<td><input type="text" id="ss'+alldata.precinctdata[i].id
					+'" value="'+alldata.precinctdata[i].scanstations+'" size=4 '+enable+'></td>';
			}
			table+='<td id="mw'+alldata.precinctdata[i].id+'">'+mw
				+'</td><td id="aw'+alldata.precinctdata[i].id+'">'+aw
				+'</td><td id="wo'+alldata.precinctdata[i].id+'">'+wo+'</td>';
			table+='</tr>';
		}
	}
	$('#precincttable').html(table);
}

// Sort table by precinct name, if not already.
function sortPrecinct()
{
	if (unsorted)
	{
		alldata.precinctdata.sort(precinctCompare); unsorted=false;
		fillTable();
	}
}

// Sort the table by max computed waiting time.
function sortWait()
{
	alldata.precinctdata.sort(waitCompare);
	unsorted=true;
	fillTable();
}
// Sort the table by avg computed waiting time.
function sortAvgWait()
{
	alldata.precinctdata.sort(avgwaitCompare);
	unsorted=true;
	fillTable();
}
// Compare precincts for sorting, descending by wait time
function waitCompare(a,b)
{
	if (parseInt(a.maxwait) < parseInt(b.maxwait)) return 1;
	if (parseInt(a.maxwait) > parseInt(b.maxwait)) return -1;
	return 0;
}
// Compare precincts for sorting, descending by avg wait time
function avgwaitCompare(a,b)
{
	if (parseInt(a.avgwait) < parseInt(b.avgwait)) return 1;
	if (parseInt(a.avgwait) > parseInt(b.avgwait)) return -1;
	return 0;
}
// Sort the table by walk-off voters
function sortWalk()
{
	alldata.precinctdata.sort(walkCompare);
	unsorted=true;
	fillTable();
}
// Compare precincts for sorting, descending by number of walk-offs
function walkCompare(a,b)
{
	if (parseInt(a.walkoffs) < parseInt(b.walkoffs)) return 1;
	if (parseInt(a.walkoffs) > parseInt(b.walkoffs)) return -1;
	return 0;
}

// One of the text values changed in the table, recompute that precinct's values.
function textc(which)
{
	// parse the id of the element that has changed. First characters are cs or vs (check-in or
	// voting column) and the remaining characters are the precinct id.
	var wid=which.id;
	// first validate that the input is a valid integer > 0
	if (validInt('#'+wid))
	{	
		var col=wid[0];
		var id=wid.substr(2);
		// find the precinct in our data that matches the one in the row being changed
		for (var i=0;i<alldata.precinctdata.length;i++)
		{
			// once the precinct is found, set the new value and rerun the simulation for this precinct
			if (alldata.precinctdata[i].id == id)
			{
				switch (col)
				{
					case 'c':
						alldata.precinctdata[i].checkinstations=which.value; break;
					case 'v':
						alldata.precinctdata[i].votingstations=which.value; break;
					case 'r':
						alldata.precinctdata[i].regstations=which.value; break;
					case 's':
						alldata.precinctdata[i].scanstations=which.value;
				}
				simOne(i);

				// if county and precinct at the top of the page match the edited row 
				// change the textbox value by the graph at the top of the page and redraw the chart
				var c = $('#county').val();
				var p = $('#precinct').val();
				if (alldata.precinctdata[i].county == c && alldata.precinctdata[i].precinct == p)
				{
					switch (col)
					{
						case 'c':
							$('#checkinstations').val(which.value); break;
						case 'v':
							$('#votingstations').val(which.value); break;
						case 'r':
							$('#regstations').val(which.value); break;
						case 's':
							$('#scanners').val(which.value);
					}
					drawChartSim(which);
				}
				break;
			}
		}
	}
}

// A precinct was selected in the option list. Populate the simulation values with data
// about this precinct and run the simulation. The "control" parameter tells which control 
// the GUI should use for the "wait" cursor (see drawChartSim function for detail).
function getPrecinctData(control)
{
	var c = $('#county').val();
	var p = $('#precinct').val();
	var html='';
	// cycle through all the data
	for (var i=0;i<alldata.precinctdata.length;i++)
	{
		// until the county and precinct match the current selection
		if (alldata.precinctdata[i].county == c && alldata.precinctdata[i].precinct == p)
		{	
			// then populate the form with this precinct's data
			var totalvoters=alldata.precinctdata[i].turnout;
			$('#voters').val(totalvoters);

			if (samedayreg) 
			{
				var regvoters=alldata.precinctdata[i].registervoters;
				$('#register').val(regvoters);
				$('#regstations').val(alldata.precinctdata[i].regstations);
			}

			$('#checkinstations').val(alldata.precinctdata[i].checkinstations);
			$('#votingstations').val(alldata.precinctdata[i].votingstations);
			if (alldata.precinctdata[i].scanstations > 0)
				$('#scanners').val(alldata.precinctdata[i].scanstations);
			
			// ****************************************************************************************
			// IGNORE THIS FIRST CONDITION. LEGACY CODE FROM WHEN WE TRIED USING MACHINE TYPE AS A 
			// FACTOR IN THE LENGTH OF TIME NEEDED TO VOTE. THE FIRST CONDITION WILL ALWAYS BE FALSE.
			// Use only if this data is available. Set m=-1 if unknown.
			var m=alldata.precinctdata[i].machine;
			// This first condition is legacy code, from when we imported Javascript data
			// created from spreadsheets that included voting machine types for each county.
			// Leaving the code in case that type of export/import is ever used again.
			if (m > -1)
			{
				var machineBrand=alldata.machines[m].brand;
				var t=alldata.machines[m].type;
				var machineTime=alldata.types[t].time;
				var machineDesc=alldata.types[t].desc;
				$('#machine').html(machineBrand+', '+machineDesc);
				$('#voteminutes').val(machineTime);
			}
			// ****************************************************************************************
			// No machine data. Is there uploaded CSV data that gives the average voting minutes
			// for this precinct?
			else if (alldata.precinctdata[i].votingminutes > 0)
			{
				$('#machine').html(' ');
				$('#voteminutes').val(alldata.precinctdata[i].votingminutes); 
			}
			// No machine or "minutes to vote" provided. Use default value.
			else
			{
				$('#machine').html(' ');
				$('#voteminutes').val(DefaultVoteMinutes); 
			}
			
			if (alldata.precinctdata[i].checkinminutes > 0)
				$('#checkinminutes').val(alldata.precinctdata[i].checkinminutes); 
			else
				$('#checkinminutes').val(DefaultCheckinMinutes); 

			if (alldata.precinctdata[i].registerminutes > 0)
				$('#regminutes').val(alldata.precinctdata[i].registerminutes); 
			else
				$('#regminutes').val(DefaultRegisterMinutes); 
				
			if (alldata.precinctdata[i].scanminutes > 0)
				$('#scanminutes').val(alldata.precinctdata[i].scanminutes); 
			else
				$('#scanminutes').val(DefaultScanMinutes); 
			
			// if on and it should be off
			if (alldata.precinctdata[i].scanstations == 0 && ($('#scannerlines').is(":checked")))
			{
				$('#scannerlines').prop('checked', false);
				$('.scanners').css('display','none');
			}
			// if off and it should be on
			else if (alldata.precinctdata[i].scanstations > 0 && !($('#scannerlines').is(":checked")))
			{
				$('#scannerlines').prop('checked', true); 
				$('.scanners').css('display','inline');
			}
					
			$('#arrivals').val(alldata.precinctdata[i].arrivalpattern);
			
			if (alldata.precinctdata[i].nocheckin > 0)
				$('#nocheckin').val(alldata.precinctdata[i].nocheckin); 
			else
				$('#nocheckin').val(DefaultCannotCheckIn);
				
			if (alldata.precinctdata[i].earlyarrival > 0)
				$('#early').val(alldata.precinctdata[i].earlyarrival); 
			else
				$('#early').val(DefaultEarlyArrivals); 
			
			break;
		}
	}
	// run and chart the simulation
	drawChartSim(control);
}

//********************************************************************************************
// FUNCTION CURRENTLY NOT IN USE
// Fill in an option list with a range of integers from min to max
function optionValues(min,max)
{ for (var i=min;i<=max;i++) document.write('<option value="'+i+'">'+i+'</option>');}
//********************************************************************************************


// Run the simulation once for each precinct with the data we have. This is triggered
// by the "Compute Wait Times" button above the table of precincts.
// NOTE: Function name is simAllCounty but it will now run the sim on ALL precincts in the dataset,
// whether currently displayed in the table or not!
function simAllCounty()
{	
	$("html").css('cursor','progress');
	$('#waitbutton').css('cursor','progress');
	setTimeout(sac,100); // this delay gives time for the cursor to switch over
} 
function sac() // Triggered after delay because of the timeout required to allow the "wait" cursor
{
	// go through all the precincts and run the simulation for each
	for (var i=0;i<alldata.precinctdata.length;i++) simOne(i);
	$("html").css('cursor','auto');
	$('#waitbutton').css('cursor','auto');
	// don't allow download until the results have all been populated... as in now.
	$('#downloadbutton').prop('disabled',false);
}

// Run the simulation once with and once without walk-offs, then run it again simulating walk-offs. 
// No charting. Just display and set max wait, avg wait, and potential walk-offs, then return.
// This function is used to update values in the precinct table without bothering with the chart. 
function simOne(i)
{
	var id=alldata.precinctdata[i].id;
	var c = $('#county').val(); // if county is currently displayed, the table will show the changes.
	
	// record the actual times used in case not provided, for download purposes
	alldata.precinctdata[i].voteminsused = (alldata.precinctdata[i].votingminutes > 0) ? alldata.precinctdata[i].votingminutes : $('#voteminutes').val();
	alldata.precinctdata[i].checkminsused = (alldata.precinctdata[i].checkinminutes > 0) ? alldata.precinctdata[i].checkinminutes : $('#checkinminutes').val();
	alldata.precinctdata[i].regminsused =  (alldata.precinctdata[i].registerminutes > 0) ? alldata.precinctdata[i].registerminutes : $('#regminutes').val();
	alldata.precinctdata[i].scanminsused =  (alldata.precinctdata[i].scanminutes > 0) ? alldata.precinctdata[i].scanminutes : $('#scanminutes').val();
	
	// Run it the first time without walk-offs to get max and avg wait time. sim returns a result object (r)
	var r=sim(alldata.precinctdata[i],false); // false means no walk-offs simulated
	alldata.precinctdata[i].maxwait=r.longestwait;
	alldata.precinctdata[i].avgwait=r.avgwait;
	if (alldata.precinctdata[i].county == c)
	{
		$('#mw'+id).html(r.longestwait);
		$('#aw'+id).html(r.avgwait);
	}
	
	// Run again computing potential walk-offs
	r=sim(alldata.precinctdata[i],true); // true to simulate walk-offs
	var w=r.walkoffs;
	alldata.precinctdata[i].walkoffs=w;
	if (alldata.precinctdata[i].county == c) 
		$('#wo'+id).html(w);
}

// drawChartSim runs the simulation using input from the page form (above and below the graph) and graphs the results.
// Cursor nonsense to show a "wait" cursor while simulation is running. Ugh.
var waitcontrol=$('#jsbutton'); // gotta remember this so we can set it back to normal. initialize so no crash.
function drawChartSim(c)
{
	$("html").css('cursor','progress');
	$(c).css('cursor','progress'); waitcontrol=c;
	setTimeout(dcs,100); // Firefox seems to need the 100ms delay. Chrome and IE were happy with 10ms.
} 
// Run the simulation with input from the page form (above and below graph) and no walk-offs. Graph the results.
function dcs(){drawChart(sim(0,0));} 


// Draw the graph. "n" contains the result arrays from the simulation.
var lastsim;
function drawChart(n)
{
	lastsim=n; // save this data in case we require a redraw of the same data set
	// This will tell if we have to clear the chart entirely before redrawing. Certain conditions below
	// set this to true if chart needs to be redone from scratch.
	var changed = false; 

	// Display walk-offs? 
	if($('#walkoffs').is(":checked"))
	{
		var wpercent = Math.round(n.walkoffs/$('#voters').val()*100,1);
		$('#walkoffdisplay').html(n.walkoffs+' walk-offs projected ('+wpercent+'%)');
	}
	else
		$('#walkoffdisplay').html('');
		
	// Some variables we'll keep between function calls so we can recall the previous state.
	// These are 'static' variables in Javascript. Create them on first call of drawChart (if 'undefined').
	if (typeof drawChart.pollOpens == 'undefined') 
	{
		drawChart.pollOpens=0; drawChart.pollCloses=0; 
		drawChart.showPattern=false; drawChart.showActual=false; drawChart.showRange=false; 
		drawChart.showSameDayReg=false;
	}
	
	// Here are a couple changes that would require redrawing and re-labeling the entire chart
	var v = parseFloat($('#opentime').val());
	if (v != drawChart.pollOpens) {drawChart.pollOpens=v; changed=true;}
	
	v = parseFloat($('#closetime').val());
	if (v != drawChart.pollCloses) {drawChart.pollCloses=v; changed=true;}
	
	// getTimeLabels returns an array of time labels for the x-axis of the chart
	var timeLabels = getTimeLabels(drawChart.pollOpens,drawChart.pollCloses);
	var starthour = Math.floor(drawChart.pollOpens);
	var startfracthour = drawChart.pollOpens - starthour;
	
	// for checking the percentages per interval if in debug mode
	var debugHTML = '<table>';
	var aSum = 0;
	
	// This next section constructs the tooltips from the data and arrival data arrays 
	// in case arrival rates are requested as part of the graph.
	var votingtips = new Array();
	narrivals = new Array(); // these are *100 for percent display
	nactual = new Array();
	
	var scanners=($('#scannerlines').is(":checked"));
	var maxpercent = 0;
	for (var i=0; i<n.voting.length; i++)
	{
		// Tooltip time display includes quarter hours.
		var hour=Math.floor(i/4)+starthour;
		var minute=((i+startfracthour*4)%4)*15;
		if (hour > 12) hour -= 12;
		var time= (minute==0) ? String(hour)+':00' : String(hour)+':'+String(minute);
	
		var totalwait = n.voting[i]+n.checkin[i]+n.scan[i];
		var scanString = scanners? 'Scan: '+waitTime(n.scan[i])+'<br>': ''; 
		votingtips.push('<b>'+time+' Avg. Wait</b><br>Check-in: '+waitTime(n.checkin[i])
			+'<br>Voting: '+waitTime(n.voting[i]) +'<br>'+scanString+'Total: '+waitTime(totalwait));

		// more debug code
		debugHTML += '<tr><td>'+i+'</td><td>'+n.arrivals[i]+'</td></tr>';
		aSum += n.arrivals[i];
		
		// scale the arrival percentage so it will show up on the chart
		// compute the maximum arrival percentage value for labeling the right side of the chart
		narrivals[i] = n.arrivals[i] * 100;
		nactual[i] = n.actual[i] * 100;
		if (narrivals[i] > maxpercent) maxpercent = narrivals[i];
		if (nactual[i] > maxpercent) maxpercent = nactual[i];
	}
	// done with tooltips and arrival arrays
	maxpercent = Math.ceil(maxpercent);

	// a couple more debug lines
	debugHTML += '</table><p>Sum is: '+aSum+'</p>';
	if (debug) $('#debug').html(debugHTML);

	// More changes that would require clear and redraw
	v = $('#showcurve').is(":checked");
	if (v != drawChart.showPattern) {drawChart.showPattern=v; if (!v) changed=true;}
	v = $('#showactual').is(":checked");
	if (v != drawChart.showActual) {drawChart.showActual=v; if (!v) changed=true;}
	v = $('#showrange').is(":checked");
	if (v != drawChart.showRange) {drawChart.showRange=v; if (!v) changed=true;}

	// Now set all the actual chart options
	var chartoptions = {template:'mine',labels:timeLabels,tooltips:votingtips};

	// serie1 through serie8 are the data series that will be plotted.
	chartoptions.values = {serie1:n.checkin, serie2:n.voting, serie3:n.scan};
	if (drawChart.showPattern) chartoptions.values.serie4=narrivals;
	if (drawChart.showActual) chartoptions.values.serie5=nactual;

	chartoptions.axis = {r: {max:maxpercent}};
	if (drawChart.showRange) {chartoptions.values.serie6=n.min;chartoptions.values.serie7=n.max;}

	if (samedayreg != drawChart.showSameDayReg) {drawChart.showSameDayReg=samedayreg; if (!samedayreg) changed=true;}
	if (samedayreg) chartoptions.values.serie8=n.register;

	// and call the ElyCharts function to actually draw the chart.
	if (changed) // if certain changes noted above, need to clear chart first
		$("#chart").show().chart("clear").chart(chartoptions);
	else
		$("#chart").show().chart(chartoptions);

	// Return the cursor to normal
	$("html").css('cursor','auto');
	$(waitcontrol).css('cursor','auto');
	//alert(n.longestwait);
}

// Create an array of readable am/pm time labels for the x-axis of the graph
// based on opening and closing times
function getTimeLabels(open,close)
{
	var labels = new Array();
	for (var time = open; time<=close; time+=.25)
	{
		var hour = Math.floor(time);
		if (time-hour > 0) 
			labels.push(''); // only label on the hour
		else
		{
			var h = hour > 12 ? String(hour-12):String(hour);
			h = hour > 11 ? h+'pm':h+'am';
			labels.push(h);
		}
	}
	return labels;
}


// Convert wait times to minutes notation, or hours if >= 120 minutes
function waitTime(min)
{
	if (min < 120)
		return String(min)+' min.';
	else
		return String(Math.round((min/60)*10)/10) + ' hrs.';
}


// This is the template definition for ElyCharts. Some variations are commented out instead
// of deleted. 

$.elycharts.templates['mine'] = 
{
	type: "line",
	margins : [10, 50, 20, 50],
	series : {
		serie1 : { // series 1 is average wait times for all voters, check-in + registration waits
			color : "yellow", 
			fill:true 
		},
		serie2 : { // series 2 is wait times for voting. note that it's stacked on top of series 1
			color : "violet", 
			stacked:true, 
			fill:true
		}, 
		serie3 : { // Series 3 is scanner waits (if used), stacked on series 1 and series 2
			color : "#44b", 
			stacked:true, 
			fill:true
		},
        serie4: { // Series 4 is the arrival percentage in each period, from the arrival pattern used
            axis: 'r',
			fill: false,
			stacked:false,
            color: "#888", //"#F80",
            plotProps: {
                "stroke-width": 2
            }
        },
        serie5: { // Series 5 is the actual randomized percentages in each period generated by the simulation
            axis: 'r',
			fill: false,
			stacked:false,
            color: "#0F0",
            plotProps: {
                "stroke-width": 2
            }
        },
        serie6: { // Series 6 and 7 are the dashed lines for min waits and max waits
			fill: false,
			stacked:false,
            color: "violet",
            plotProps: {
                "stroke-width": 2,
				"stroke-dasharray":"-"
            }
        },
        serie7: {
			fill: false,
			stacked:false,
            color: "violet",
            plotProps: {
                "stroke-width": 2,
				"stroke-dasharray":"-"
            }
        },
        serie8: { // series 8 is the registration waits, plotted separately in red. if the red line appears above
		// the yellow check-in + registration line, registration waits are the longer hold up. If the red line is 
		// lower, the check-in stations cause the longer delay.
			fill: false,
			stacked:false,
            color: "#F00",
            plotProps: {
                "stroke-width": 2
            }
        }
	},
	defaultAxis : {
	  labels : true, max:120, normalize: 3
	},
    axis: {
        l: { // left axis 
			//title : "Wait time in minutes" ,
			// titleDistanceIE:4, // still an issue with IE, so omit this title
			//titleDistance : 35,
			max:120, 
			normalize: 3,
            labels: true,
			labelsFormatHandler: function(label) {return label+' min'},
            //labelsDistance: 0,
            labelsSkip: 1
        },
        r: { // right axis
			//title: "Percent of total arrivals" ,
			//titleDistance : 35,
			max:4, 
			normalize: 3,
            labels: true,
			labelsFormatHandler: function(label) {return label+'%'},
            //labelsDistance: 0,
            labelsSkip: 1
        }
    },
	defaultSeries: { 
		fill: true, 
		stacked:true,
		rounded: true,
		plotProps : {
		   "stroke-width" : 2
		},
		tooltip : {
			//frameProps : false,
			frameProps : {
				stroke : "gray"
			},
			height : 88, width: 120,
			offset : [50, 50]   
			// first number is y, and - moves down, + moves up. second number - moves right!
			//contentStyle : "font-weight: bold"
		
		},
		startAnimation:{
			active:true,
			speed : 600,
			delay : 0,
			type : "reg",
			easing : "<"
		},
		stepAnimation:{
			active:true,
			speed : 600,
			delay : 0,
			type : "reg",
			easing : "<>"
		
		}

	},
	features : {
		grid : {
			draw : true,
			forceBorder : true,
			ny: 8
		},
		
        mousearea: {
            type: 'axis'
        },
		tooltip : {
			fixedPos:[80, 20]
		}
        //tooltip: {
        //    positionHandler: function(env, tooltipConf, mouseAreaData, suggestedX, suggestedY) {
        //       return [mouseAreaData.event.pageX-50, mouseAreaData.event.pageY-150, true]
        //}}
    }
	
}


//**************************************************************************************************
// HERE'S THE ACTUAL SIMULATION SECTION!
//**************************************************************************************************

// Timing constants:
var TICKSPERMINUTE = 6; // advance through the day in 10-second "ticks"
var MINUTESPERINTERVAL = 15; // most of the data collected for arrival rates arrived aggregated in 15-minute segments
var INTERVALSPERHOUR = 60 / MINUTESPERINTERVAL; // yeah, it's 4
var TICKSPERINTERVAL = TICKSPERMINUTE * MINUTESPERINTERVAL; // 15 * 6 = 90 10sec "ticks" per 15 minute interval

// Here are the regression coefficients from our samples
// These are based on 52 intervals (1-52) and f(1)+f(2)+...f(52) = 1.00 or 100%
var coeffs = new Array(
// composite
 new Array(0.00000000044918840,-0.00000007509575190,0.00000473958964919,
	-0.00014038819829617,0.00197813342612455,-0.01179658708140930,0.03698977546993090),
// WIjen morning, mid, + after work
 new Array(0.00000000023758320,-0.00000004001070799,0.00000251258483600,
	-0.00007254345567801,0.00096078045702784,-0.00504055767236753,0.02456103770367880),
// c117 small morning + after work (similar to c203)	
 new Array(-0.00000000036328206,0.00000005346954060,-0.00000303505326454,
	0.00008409537686300,-0.00114900879988595,0.00620927084993522,0.01342388651520370),
// WIbob mid-morning + after work, steadier
 new Array(0.00000000018855216,-0.00000003338513963,0.00000219878582700,
	-0.00006682681898892,0.00093928250610031,-0.00519523312672686,0.02459205503884530),
// c201 mid-morning + after work, midday lull
 new Array(0.00000000017310322,-0.00000003353314546,0.00000234986645500,
	-0.00007371444590081,0.00102405684158150,-0.00522800290212594,0.02321669437533050),
// WI253 steady + evening peak
 new Array(-0.00000000370670400,0.00000040556513821,-0.00001484586194638,
	0.00021674952531368,-0.00111455434320653,0.01656403063257890),
// c302 evening peak
 new Array(-0.00000000022466094,0.00000002998677775,-0.00000154222191577,
	0.00004074309285221,-0.00060317478521214,0.00453804967452955,0.00213387272172039),
// c103 morning peak
 new Array(0.00000000400742928,-0.00000061748072527,0.00003352733617507,
	-0.00075031663656100,0.00581637965339026,0.01479665946133670),
// v11f mid morning peak
 new Array(0.00000000022198343,-0.00000004176253557,0.00000294640087255,
	-0.00009676858594411,0.00146971054127016,-0.00855818578443484,0.02970311716134730),
// flat
 new Array(-0.00000000018324512,0.00000002951042655,-0.00000185493448738,
	0.00005738347432461,-0.00090269468034732,0.00665179970329544,0.00277714932126771)
);
for (var i=0;i<coeffs.length;i++)
	coeffs[i].reverse(); // ack. pasted the above results from excel, but they're in reverse order.


// Given an array of coefficients c[0]*x^0 + c[1]*x^1 + c[2]*x^2...
// evaluate the polynomial with value x
function polynomial(x,c)
{
	var y=0;
	for (var e = 0; e < c.length; e++)
		y += c[e]*Math.pow(x,e);
	return y;
}

///////////////////////////////  THE SIMULATION CODE //////////////////////////////////////////////////////////////////////////
// data=0 means use the data from the page form, else data is object containing the parameters (used when running all precincts)
// datawalkoffs is boolean, true means use walkoff calculations
function sim(data,datawalkoffs)
{
	// Defininition of a Voter class, used within sim().
	// Create a new voter. timeParam has 3 different meanings:
	// 0 = this is an early arrival. assign arrival time = 0 and interval = 0
	// < 100 = this is an INTERVAL number (starting at 1). We're working on 15-minute intervals, so a 13-hour day
	//      will have intervals numbered 1 to 52. Randomly assign an arrive time within this 15-minute interval
	// > 100 = this is the total number of "ticks" in the voting day. pick a random number (0 to 1) and compare to
	//      the distribution map to determine arrival time, then compute interval number.
	function Voter(timeParam)
	{
		this.interval = ''; // integer representing the 15-minute interval to which this voter belongs, starting at 0
		
		// the following are all integers representing a number of "ticks" since polls opened (t=0 is opening time)
		this.arrival = ''; // absolute arrival time in ticks. need to sort by this value for queuing up. value is
			// set to t=0 for those waiting when polls open.
		this.checkintime = ''; // absolute check-in time in ticks
		this.checkedin = ''; // finished check-in, now waiting to vote
		this.votetime = ''; // now starting to vote
		this.voted = ''; // finished voting, may need to wait for scanner
		this.scantime = ''; // starting to scan
		this.scanned = ''; // finished scanning
		
		this.checkinwait = ''; // number of ticks between arrival and checking in
		this.votewait = ''; // number of ticks between checked in and voting
		this.scanwait = ''; // number of ticks between voted and starting to scan
		
		if (timeParam == 0) // early arrival, set arrival and interval both to 0
		{
			this.arrival = 0;
			this.interval = 0;
		}
		else if (timeParam < 100) // use this as the interval number
		{
			this.interval = timeParam;
			// assign a tick value for arrival time, = ticks for all previous intervals + random tick within this interval
			this.arrival = (timeParam-1) * TICKSPERINTERVAL + Math.random()*TICKSPERINTERVAL + 1; // 1 to TICKSPERINTERVAL
		}
		else // pick a random number 0 to 1 and use distribution curve to find arrival time, 
		// then compute which interval that falls in
		{
			var r = Math.random();
			for (var d=0;d<=timeParam;d++)
				if (r < distribution[d])
				{ 
					this.arrival = d;
					this.interval = Math.ceil(d/TICKSPERINTERVAL); 
					break;
				}
		}
		
		// Decide whether this is one of the voters for same-day reg line.
		// Decrement one or both parameters to change the odds for each successive voter
		// and assure that exactly the number of voters projected to register
		// are generated.
		this.registerMe = false;
		if (Math.random() < registerRemaining/votersRemaining)
		{
			this.registerMe = true;
			registerRemaining--; 
		}
		votersRemaining--;
		
		this.computeCheckWait = function()
		{ this.checkinwait = this.checkintime - this.arrival; }
		
		this.computeVoteWait = function()
		{ this.votewait = this.votetime - this.checkedin; }
		
		this.computeScanWait = function()
		{ this.scanwait = this.scantime - this.voted; }

	}
	///////////// Voter instance construction completed //////////////////////////
	
	// compare two Voters by arrival time, for sorting
	function votercompare(a,b)
	{
		return (a.arrival - b.arrival);
	}


	//////////// Back to the simulation code now //////////////////////////////////

	// Using JQuery notation to grab all the required parameters from the page form
	var votetimeminutes = $('#voteminutes').val(); // set defaults, will change if provided
	var checkintimeminutes = $('#checkinminutes').val();
	var regtimeminutes = $('#regminutes').val();
	var scantimeminutes = $('#scanminutes').val();
	
	// This is the percentage of the day's voters that are already 
	// waiting in line when the polls open, converted to decimal form
	var earlyarrivalpercent = DefaultEarlyArrivals/100; 
	if ($('#early').length > 0) earlyarrivalpercent = $('#early').val()/100;
	
	var checkbutnovotepercent = $('#nocheckin').val();

	var scanners=false;
	if (data==0) // grab the data from the page form
	{
		var votercount = $('#voters').val();
		var checkinstations = $('#checkinstations').val();
		var votestations = $('#votingstations').val();
		scanners=($('#scannerlines').is(":checked"));
		var scanstations = $('#scanners').val();
		if (samedayreg)
		{
			var registercount = $('#register').val();
			var regstations = $('#regstations').val();
		}
		else
			var registercount=0, regtimeminutes=0, regstations=0;
		
		var pattern = $('#arrivals').val();
		var randomDay = ($('[name="randomize"]:checked').val() == 'day'); // JQuery notation for radio buttons
		var numberofsims = $('#repetitions').val();
		// compute walk-offs in simulation?
		var walkoffs = $('#walkoffs').is(":checked");
	}
	else // one-time run with data we're feeding in via data variable
	{
		var votercount = data.turnout;
		var checkinstations = data.checkinstations;
		var votestations = data.votingstations;
		if (samedayreg)
		{
			var registercount = data.registervoters;
			var regstations = data.regstations;
		}
		else
			var registercount=0, regtimeminutes=0, regstations=1; // don't use zero, or we'll get a division error below
			
		if (data.scanstations == 0)
			var scanstations=1; // avoid divide by zero error
		else
		{
			scanners=true;
			var scanstations=data.scanstations;
		}
		
		votetimeminutes = data.voteminsused;
		checkintimeminutes = data.checkminsused;
		regtimeminutes = data.regminsused;
		scantimeminutes = data.scanminsused;
		var pattern=data.arrivalpattern;
		if (data.earlyarrival >= 0) earlyarrivalpercent=data.earlyarrival/100;
		if (data.nocheckin >= 0) checkbutnovotepercent=data.nocheckin;
		
		var randomDay=false; // Closely use the arrival pattern in one simulation run
		var numberofsims=1;
		var walkoffs=datawalkoffs;
	}

	

	var open = $('#opentime').val();
	var close = $('#closetime').val();
	var votinghours = parseFloat(close) - parseFloat(open);
	
	// computed from above
	var checkinticks = checkintimeminutes * TICKSPERMINUTE;
	var registerticks = regtimeminutes * TICKSPERMINUTE;
	var voteticks = votetimeminutes * TICKSPERMINUTE;
	var scanticks = scantimeminutes * TICKSPERMINUTE;
	var totalintervals = votinghours * INTERVALSPERHOUR;
	
	var arrivalPercent = new Array();
	arrivalPercent[0]=earlyarrivalpercent;
	
	// This loop is computing the projected percentage of voters who will arrive during 
	// any interval (currently defined as 15 minutes). This is used to display the arrival
	// curve used (although random distribution may vary for each trial). It is also used
	// as the exact percentage if the "arrivals follow pattern" option is selected instead
	// of the default "arrivals random based on pattern".
	var xval=0.5;
	// The polynomial curve is based on 52 intervals. Scaling "incval" (increment value) 
	// adjusts it if we are using a different number of intervals in this run of the simulation.
	var incval = 52/totalintervals; 
	for (var i=1;i<=totalintervals;i++)
	{
		// evaluate the polynomial at this interval value and scale result for early arrival percentage
		// and difference in number of intervals, if necessary (incval)
		var j = polynomial(xval,coeffs[pattern])/(1+earlyarrivalpercent)*incval;
		xval+=incval;
		arrivalPercent[i]=j;
	}
	
	// Round off errors in the polynomial evaluations will result in a cumulative percentage that is not
	// quite 100%. Compute the total, then normalize the arrivalPercent values so the result is 1.
	var normalize=0;
	for (i=0;i<=totalintervals;i++) normalize+=arrivalPercent[i];
	for (i=0;i<=totalintervals;i++) arrivalPercent[i]/=normalize;
	
	// These will be used at the end for computing average wait times. Inititalize now so that
	// we can run the sim multiple times and average the averages.
	var checkinwaits = new Array();
	var registerwaits = new Array();
	var registertotals = new Array();
	var votingwaits = new Array();
	var scanwaits = new Array();
	var didnotvote = new Array();
	var walkoff = new Array();
	var maxwaits = new Array();
	var minwaits = new Array();
	var arrivaltotals = new Array();
	for (i=0; i <= totalintervals; i++)
	{
		checkinwaits[i]=0;
		registerwaits[i]=0;
		registertotals[i]=0;
		votingwaits[i]=0;
		scanwaits[i]=0;
		didnotvote[i]=0;
		walkoff[i]=0;
		maxwaits[i]=0;
		minwaits[i]=99999;
		arrivaltotals[i]=0; // for multiple runs, keep track of total arrivals within interval
	}
	// These two variables for computing mean wait time for ALL those who voted
	var waittickstotal=0;
	var totalvoted=0;

	// Work out distribution probability (by tick) based on polynomial...
	// 13 hours is 46800 seconds
	var totalticks = totalintervals * TICKSPERINTERVAL;
	var distribution = new Array(); // omit the var to make this global

	// This computes the incremental percentage of arrivals for each TICK (currently defined
	// as 10 seconds). Distribution is used when randomly generating voters to determine each 
	// voter's arrive time at a precise "tick". This array is used in the Voter constructor
	// when randomly placing a voter by probability throughout the day.
	distribution[0]=earlyarrivalpercent;
	xval=0.5;
	incval=52/totalticks;
	for (var d=1; d<=totalticks; d++)
	{
		distribution[d] = distribution[d-1]+polynomial(xval,coeffs[pattern])/(1+earlyarrivalpercent)*incval;
		xval+=incval;
	}
	
	// Round off errors in the polynomial evaluations will result in a cumulative percentage that is not
	// quite 100%. Normalize the distribution values so the result is 1 at the end of the day.
	for (i=0;i<=totalticks;i++) distribution[i]/=distribution[totalticks];
	
	// For repeating the simulation multiple times
	for (var sims = 1; sims <= numberofsims; sims++)
	{

		var arrivalqueue = new Array(); // actual queue of Voters
		var arrivals = new Array(); // how many arrivals per interval
		var registerRemaining = registercount; // Used for each sim run to select voters to register
		var votersRemaining = votercount;
		// (IF samedayreg is true!) Each new voter uses registercount/votersremaining for probability
		// of assigning new voter as one needing to register.

		if (randomDay) // random arrivals using distribution curve for random placement
		{
			// now populate the arrival queue with voters
			for (i = 0; i < votercount; i++)
				// new voter. arrival time will be random based on distribution values.
				arrivalqueue.push(new Voter(totalticks)); 
				
			// make sure the voters are arranged in order of arrival
			arrivalqueue.sort(votercompare);

			// how many voters in each interval group? this is used for averaging wait times
			for (i=0; i <= totalintervals; i++)
				arrivals[i]=0;
			for (i = 0; i < votercount; i++)
				arrivals[arrivalqueue[i].interval]++;
		}
		
		else	// compute arrivals per interval based on distribution function, THEN randomize within interval
		{
			// Use the percentages for each interval to assign the number of arrivals per interval
			var roundoff=0;
			for (var i = 0; i <= totalintervals; i++)
			{
				// need to make sure the rounding doesn't lose voters in aggregate
				var floated = votercount * arrivalPercent[i] + roundoff;
				arrivals[i] = Math.round(floated);
				roundoff = floated-arrivals[i]; // +if rounded down, -if rounded up 
			}
			
			// Now that we know how many voters per interval, randomize their arrival
			// times WITHIN that interval
			for (i = 0; i <= totalintervals; i++)
				for (var a = 0; a < arrivals[i]; a++)
					arrivalqueue.push(new Voter(i)); // create a new voter in interval i, with random arrival time within interval
					
			// and make sure they're arriving in order...
			arrivalqueue.sort(votercompare);
		}


		// READY TO ROCK! LET'S OPEN THE POLLS!

		// set up the polling-place queues:
		// arrivalqueue->waitingtocheckin/waitingtoregister->checkingin/registering->waitingtovote->votingnow->finishedvoting
		// (with options added for waitingtoregister->registering and waitingtoscan->scanningnow)
		var waitingtocheckin = new Array();
		var waitingtoregister = new Array();
		var checkingin = new Array(); // those in the check-in process
		var registering = new Array(); 
		var waitingtovote = new Array(); // checked in, waiting to vote
		var votingnow = new Array(); // in the process of voting
		var waitingtoscan = new Array();
		var scanningnow = new Array();
		var finishedvoting = new Array(); // they've got their stickers

		var time = 0; // our 10-second clock... each tick is 10 seconds.

		// AND WE'RE OFF!
		// Main loop continues until all the queues (except finishedvoting) are empty. We'll process voters after closing
		// time until everyone is finished. (All voters are generated such that they arrive before closing time.)
		while (arrivalqueue.length > 0 || waitingtocheckin.length > 0 || waitingtoregister.length > 0 
			|| checkingin.length > 0 || registering.length > 0 || waitingtovote.length > 0 || votingnow.length > 0
			|| waitingtoscan.length > 0 || scanningnow.length > 0)
		{
			// So now we move voters through each of the queues. Logically, we work our way backwards and clear any 
			// stations where processing is completed first. That way, for example, if one voter has finished voting 
			// and we move them out of a voting station, the next step looks back to the "waiting to vote" line and 
			// will move the first person in that line to the voting station that's just been vacated. Likewise, if 
			// someone has finished checking-in and we temporarily put them in the "waiting to vote" queue, they'll 
			// immediately jump ahead to an open voting station if one is available instead of waiting for the next
			// time around the loop, 10 seconds later. (If we worked forward through the process, we'd inadvertantly 
			// add a 10-second delay to each step, as for example the voter above wouldn't move from the line to the 
			// voting station until we loop around to the next 10-second "tick".)

			// FIRST CLEAR ANY STATIONS IF ANYONE IS FINISHED ON THIS TICK
			
			// Has anyone finished scanning? (Loop, because if more than one station, more than one might finish at the same time.)
			while (scanningnow.length > 0 && scanningnow[0].scantime + scanticks <= time)
			{
				scanningnow[0].scanned = time;
				scanningnow[0].computeScanWait();
				// In Javascript, "push" adds someone to the end of a queue, and "shift" removes someone from the front 
				finishedvoting.push(scanningnow.shift());
			}
			
			// Has anyone finished voting?
			while (votingnow.length > 0 && votingnow[0].votetime + voteticks <= time)
			{
				votingnow[0].voted = time;
				votingnow[0].computeVoteWait();
				// If we're using scanners, sent them to the scanner line
				if (scanners)
					waitingtoscan.push(votingnow.shift());
				else
				{
					// don't need to scan
					votingnow[0].scantime = time;
					votingnow[0].scanned = time;
					votingnow[0].scanwait = 0;
					finishedvoting.push(votingnow.shift()); // woo hoo!
				}
			}

			// Has anyone finished checking in? if someone is now checked in...
			while (checkingin.length > 0 && checkingin[0].checkintime + checkinticks <= time)
			{
				checkingin[0].checkedin = time;
				
				// Went through check-in process, but cannot vote here? These will probably take a little more
				// time than usual, won't they? May want to add that into the sim.
				if (Math.random()*100 < checkbutnovotepercent)
				{
					checkingin[0].voted = -1;
					checkingin[0].votetime = -1;
					checkingin[0].votewait = -1;
					// throw them into the "voted" queue since they're done here
					finishedvoting.push(checkingin.shift());
				}
				else // Good to go: wait to vote now.
					waitingtovote.push(checkingin.shift()); // checked in, wait again?
			}
			
			// likewise, has anyone finished registering? if registered, get in the voting line:
			while (registering.length > 0 && registering[0].checkintime + registerticks <= time)
			{
				registering[0].checkedin = time;
				waitingtovote.push(registering.shift()); // Good to go: wait to vote now.
			}

			// NOW THAT WE'VE CLEARED ANY AVAILABLE STATIONS, CHECK THE WAITING LINES AND MOVE
			// THE FIRST PERSON IN EACH LINE TO OPEN STATIONS, IF ANY.
			
			// if there's room to scan AND someone's in the "waiting to scan" queue:
			while (scanningnow.length < scanstations && waitingtoscan.length > 0)
			{
				waitingtoscan[0].scantime = time;
				scanningnow.push(waitingtoscan.shift()); // scan it now.
			}
			
			// if there's room to vote AND someone's in the "waiting to vote" queue:
			while (votingnow.length < votestations && waitingtovote.length > 0)
			{
				waitingtovote[0].votetime = time;
				votingnow.push(waitingtovote.shift()); // time to choose!
			}
			
			// Have any new voters arrived? if so, a greeter will send them to the check-in line or 
			// the same-day reg line:
			while (arrivalqueue.length > 0 && arrivalqueue[0].arrival <= time)
			{
				var ran = Math.random(); // This random number is used if we're simulating walk-offs.
				if (arrivalqueue[0].registerMe)
				{
					// SIMULATE WALK-OFFS?
					// Compute projected register+vote+scan wait time
					var wait = Math.floor((waitingtoregister.length*registerticks/regstations
						+waitingtovote.length*voteticks/votestations
						+waitingtoscan.length*scanticks/scanstations)/TICKSPERMINUTE/30);
					// div by 30 creates buckets 0,1,2,3,etc. where 0 is < 30 min, 1 is 30-60, 2 is 60-90, 3 is >90
					if (wait > 3) wait = 3;
					if (walkoffs && (ran < WALKOFFPERCENTS[wait])) // walk away
					{
					
						arrivalqueue[0].voted = -2;
						arrivalqueue[0].votetime = -2;
						arrivalqueue[0].votewait = -2;
						arrivalqueue[0].checkintime = -2;
						arrivalqueue[0].checkinwait = -2;
						// throw them into the "voted" queue since they're done here
						finishedvoting.push(arrivalqueue.shift());
					}
					else
						// NO WALK-OFF. Just put the new person in line
						waitingtoregister.push(arrivalqueue.shift());
				}
				else
				{
					// SIMULATE WALK-OFFS?
					// Compute projected checkin+vote+scan wait time
					var wait = Math.floor((waitingtocheckin.length*checkinticks/checkinstations
						+waitingtovote.length*voteticks/votestations
						+waitingtoscan.length*scanticks/scanstations)/TICKSPERMINUTE/30);
					// div by 30 creates buckets 0,1,2,3,etc. where 0 is < 30 min, 1 is 30-60, 2 is 60-90, 3 is >90
					if (wait > 3) wait = 3;
					if (walkoffs && (ran < WALKOFFPERCENTS[wait])) // walk away
					{
						//alert('wait'+wait+'random'+ran+'ci'+waitingtocheckin.length+'vo'+waitingtovote.length);
						arrivalqueue[0].voted = -2;
						arrivalqueue[0].votetime = -2;
						arrivalqueue[0].votewait = -2;
						arrivalqueue[0].checkintime = -2;
						arrivalqueue[0].checkinwait = -2;
						// throw them into the "voted" queue since they're done here
						finishedvoting.push(arrivalqueue.shift());
					}
					else
						// NO WALK-OFF. Just put the new person in line
						waitingtocheckin.push(arrivalqueue.shift());
				}
			}
			
			// is there a check-in station ready and is someone waiting?
			while (checkingin.length < checkinstations && waitingtocheckin.length > 0)
			{
				waitingtocheckin[0].checkintime = time;
				waitingtocheckin[0].computeCheckWait();
				checkingin.push(waitingtocheckin.shift()); // can check in now!
			}
			
			// is there a registration station ready and is someone waiting?
			while (registering.length < regstations && waitingtoregister.length > 0)
			{
				waitingtoregister[0].checkintime = time;
				waitingtoregister[0].computeCheckWait();
				registering.push(waitingtoregister.shift()); // can register now!
			}
			
			time++; // another "tick" elapses... 
		}
		// END OF LOOP... AND THE DAY PROGRESSES


		// GOT HERE? THAT MEANS ALL THE QUEUES ARE EMPTY AND THE DAY IS OVER! ///////////////////////////////////
		// WORK OUT THE INTERVAL AVERAGES NOW
		// sum up the wait times for each 15-minute interval
		for (i=0; i<finishedvoting.length; i++)
		{
			var ir = finishedvoting[i].interval;
			var cw = finishedvoting[i].checkinwait;
			var vw = finishedvoting[i].votewait;
			var sw = finishedvoting[i].scanwait;
			arrivaltotals[ir]++;
			
			// Check for condition for walk-off (cw=-2)
			if (cw >= 0)
			{
				// Not a walk-off
				checkinwaits[ir] += cw; // every voter's wait for either check-in or same-day registration

				// Check condition for check-in/no vote situation (vw=-1)
				if (vw >= 0)
				{
					votingwaits[ir] += vw;
					scanwaits[ir] += sw;
					
					// keep totals for computing the mean wait time for all who voted
					waittickstotal +=(vw+cw+sw);
					totalvoted++;
					
					// keep track of the longest and shortest individual wait in the time period, in minutes
					var tm = Math.ceil((vw+cw+sw)/TICKSPERMINUTE); // total wait minutes
					if (tm > maxwaits[ir]) maxwaits[ir]=tm;
					if (tm < minwaits[ir]) minwaits[ir]=tm;
				}
				else  // -1 means this person waited to check-in but was not allowed to vote here
					didnotvote[ir]++;
				
				// total the registration waits separately from checkinwaits, too.
				if (finishedvoting[i].registerMe)
				{
					registerwaits[ir]+=cw;
					registertotals[ir]++;
				}
			}
			else 
				walkoff[ir]++; // -2 means walk-off due to long lines
		}
	} 
	// THIS IS THE END OF THE SIMULATION LOOP. REPEAT IF MULTIPLE SIMS DESIRED.


	// TIME TO AGGREGATE ALL THE COLLECTED DATA FROM THE MULTIPLE SIMULATION RUNS
	// Compute the average wait times for each interval in minutes
	var totalwalkoffs = 0;
	var totalnocheckin = 0;
	var longestavgwait = 0;
	var walkoffdebug = ''; // For development purposes only
	for (i=0; i <= totalintervals; i++)
	{
		if (minwaits[i]>10000) minwaits[i]=0; // if none in this interval, minwaits never changed from 99999
		
		totalwalkoffs += walkoff[i];
		totalnocheckin += didnotvote[i];
		walkoffdebug = walkoffdebug +':'+String(walkoff[i]); // For development only, to show walk-offs per interval
		
		if (arrivaltotals[i]-walkoff[i]>0)
		{
			checkinwaits[i] = Math.round(checkinwaits[i]/(arrivaltotals[i]-walkoff[i])/TICKSPERMINUTE,1); //
			// subtraction is for check-in/no-vote situation
			if (arrivaltotals[i]-didnotvote[i]-walkoff[i] > 0)
			{
				votingwaits[i] = Math.round(votingwaits[i]/(arrivaltotals[i]-didnotvote[i]-walkoff[i])/TICKSPERMINUTE,1); //
				scanwaits[i] = Math.round(scanwaits[i]/(arrivaltotals[i]-didnotvote[i]-walkoff[i])/TICKSPERMINUTE,1); 
			}
			else
			{
				votingwaits[i] = 0;
				scanwaits[i] = 0;
			}
		}
		else
		{
			checkinwaits[i] = 0;
			votingwaits[i] = 0;
			scanwaits[i] = 0;
		}
		
		// compute the percentage of actual arrivals during this interval, in case it's requested on the graph
		arrivaltotals[i]/= (votercount*numberofsims);
		
		// same day reg subset
		if (registertotals[i]>0)
			registerwaits[i] = Math.round(registerwaits[i]/(registertotals[i])/TICKSPERMINUTE,1);
		else
			registerwaits[i]=0;
		
		var totwait = checkinwaits[i]+votingwaits[i]+scanwaits[i];
		if (totwait>longestavgwait) longestavgwait = totwait;
	}

	// average these values for the multiple simulation runs
	totalwalkoffs=Math.round(totalwalkoffs/numberofsims,1);
	totalnocheckin=Math.round(totalnocheckin/numberofsims,1);
	
	// Compute mean wait time for ALL those who voted
	var averagewait=0;
	if (totalvoted>0) averagewait = Math.round(waittickstotal/totalvoted/TICKSPERMINUTE,1);
	
	//alert(walkoffdebug);
	
	// FINALLY, RETURN ALL THE RESULTING DATA ARRAYS, ETC.
	return {checkin: checkinwaits, voting: votingwaits, scan: scanwaits,
		arrivals: arrivalPercent, actual: arrivaltotals, 
		register: registerwaits, min: minwaits, max: maxwaits, walkoffs: totalwalkoffs, 
		nocheckin: totalnocheckin, longestwait: longestavgwait, avgwait: averagewait};
}
///////////////// DONE WITH SIMULATION //////////////////////////////////////////////////////////



//////////////// UPLOAD AND DOWNLOAD FUNCTIONS //////////////////////////////////////////////////
// Got a filename, so try to load and parse the data from the local file.
function uploadData()
{
	var f = document.getElementById('fileinput').files[0];
	if (f)
	{
		var n=f.name;
		var i=n.lastIndexOf('.');
		var ext= (i>0&&i<n.length-2) ? n.substr(i+1) : ''; // extract the file extension
		
		if (f.type != 'text/plain' && ext != 'csv') // need a plain text or csv file
		{
			alert("Please choose a CSV or tab-delimited text file, or Cancel.") 
		}
		else
		{
			closeUploadDialog();
			var r = new FileReader(); // Set Javascript to read a local file
			r.onload = function(e) // This function is called after r.readastext (the command after function def) executes!
			{ 
				var contents = e.target.result;
				//alert( "Got the file.n" +"name: " + f.name + "\n" +"type: " + f.type + "\n" +"size: " + f.size + " bytes\n"
				//	+ "starts with: " + contents.substr(0, contents.indexOf("\n")));
				
				var lines = e.target.result.split('\n'); // split the file into lines
				
				// Start by processing the headings line. Identify the headings that we recognize.
				if (ext=='csv')
					var headings = lines[0].split(','); // split lines in csv files around the commas
				else
					var headings = lines[0].split('\t'); // split line in tab-delimited files around the tabs
					
				// As we find each heading, we'll assign the column index of that heading to one of these variables.
				// This way the order and number of headings does not matter to us.
				var c=99; //county (99 is greater than number of fields. see below for usage)
				var p=-1; //precinct
				var t=99; //turnout 
				var r=99; //registration (same-day)
				var vs=99; // voting stations
				var cs=99; // check-in stations
				var rs=99; // same-day registration stations
				var ss=99; // scanner stations
				var vm=99; // minutes to vote
				var cm=99; // minutes to check in
				var rm=99; // minutes to register
				var sm=99; // minutes to scan
				var ap=99; // arrival pattern
				var nc=99; // cannot check in % (no check-in)
				var ea=99; // early arrival %
				var m=99; //voting machine type... not used, but here so that placeholder gets set to -1 below
				
				/* FOR FUTURE ADDITION?
				var v=99; // registered voters
				var vvs=99; // registered voters per vote station
				var vcs=99; // reg voters per check-in station
				var vrs=99; // reg voters per reg station
				var br=99; // races on the ballot
				var bi=99; // initiatives and referendums on the ballot
				*/
				
				// If we don't have a "alldata" object for multiple precincts yet, create it.
				if (typeof alldata === 'undefined') alldata = new Object();
				
				$('#samedayregistration').prop('checked', false); // off, unless there is a column with numbers for this
				samedayreg = false;
				$('.samedayreg').css('display','none');
				
				// In the first row, find which column headings exist, and in which column
				for (var i=0; i<headings.length; i++)
				{
					// leading/trailing whitespace doesn't matter; UPPER/lower doesn't matter; spaces,-,_,",and. don't matter
					var h = headings[i].trim().toLowerCase().replace(/[ -_."]/g,'');
					if (h=='precinct') p=i;
					else if (h=='turnout') t=i;
					else if (h=='county') c=i;
					else if (h=='registration') 
					{
						r=i; 
						samedayreg=true;
						$('#samedayregistration').prop('checked', true);
						$('.samedayreg').css('display','inline');
					}
					else if (h=='votingstations') vs=i;
					else if (h=='checkinstations') cs=i;
					else if (h=='registrationstations') rs=i;
					else if (h=='minutestovote') vm=i;
					else if (h=='minutestocheckin') cm=i;
					else if (h=='minutestoregister') rm=i;
					else if (h=='minutestoscan') sm=i;
					else if (h=='scanners') ss = i;
					else if (h=='arrivalpattern') ap=i;
					else if (h=='cannotcheckin') nc=i;
					else if (h=='earlyarrival') ea=i;
					
					/*
					else if (h=='registered voters') v=i; // only relevant for the next 3, but could be used to calc turnout
					else if (h=='voters per voting station') vvs=i; // these three may not be used. best to
					else if (h=='voters per check-in station') vcs=i; // put these calcs in their spreadsheet
					else if (h=='voters per registration station') vrs=i; // and use estimates based on turnout?
					
					else if (h=='machine type') m=i; // these three can be used for time to vote computation
					else if (h=='races') br=i;
					else if (h=='referendums') bi=i;
					*/
				}

				// if no precinct column found, abort. it's the only required one.
				if (p<0)
					alert('Did not find a column with "Precinct" in the heading row.');
				else
				{
					// remove previous data and start with a new precinct data array
					if (typeof alldata !== 'undefined')
					{
						delete alldata.precinctdata;
						delete alldata.columns; // records which columns were imported
					}
					alldata.precinctdata = new Array();
					// remember which columns were included in import; we'll use this to determine which to export
					alldata.columns = {c:c,p:p,t:t,r:r,vs:vs,cs:cs,rs:rs,ss:ss,vm:vm,cm:cm,rm:rm,sm:sm,ap:ap,ea:ea,nc:nc};
					
					// Now go through the rest of the rows and grab data from the marked columns
					for (i=1; i<lines.length; i++)
					{
						var pr = new Object();
						if (ext=='csv') // split the line by commas (CSV) or tabs (TXT)
							var line=lines[i].replace(/"/g,'').split(','); // won't work properly if comma in the actual data
						else
							var line=lines[i].split('\t');
							
						if (line.length>p && line[p].length>0) // if index p within line and column not empty
						{
							pr.precinct=line[p].trim();
							pr.id=i; // save a numeric id for matching elsewhere in the program
							
							// for each datum, check to see if enough columns to avoid index error,
							// then make sure provided value is valid
							pr.turnout = (line.length>t) ? line[t].trim() : DEFAULTTURNOUT;
							if (notInt(pr.turnout)) pr.turnout= DEFAULTTURNOUT;
							
							pr.county = (line.length>c) ? line[c].trim() : 'County';
							if (pr.county.length < 1) pr.county = 'County';
							
							pr.registervoters = (line.length>r) ? line[r].trim() : 0;
							if (notInt(pr.registervoters)) pr.registervoters= 0;
							
							pr.machine = (line.length>m) ? line[m].trim() : -1; // unknown machine type
							if (notInt(pr.machine)) pr.machine= -1;
							
							pr.checkinstations = (line.length>cs) ? line[cs].trim() : Math.ceil(pr.turnout/VOTERSPERCHECKSTATION);
							if (notInt(pr.checkinstations)) pr.checkinstations= Math.ceil(pr.turnout/VOTERSPERCHECKSTATION);
							
							pr.votingstations = (line.length>vs) ? line[vs].trim() : Math.ceil(pr.turnout/VOTERSPERVOTESTATION);
							if (notInt(pr.votingstations)) pr.votingstations= Math.ceil(pr.turnout/VOTERSPERVOTESTATION);
							
							if (samedayreg)
							{
								pr.regstations = (line.length>rs) ? line[rs].trim() : Math.ceil(pr.registervoters/VOTERSPERREGSTATION);
								if (notInt(pr.regstations)) pr.regstations= Math.ceil(pr.registervoters/VOTERSPERREGSTATION);
							}
							
							pr.scanstations = (line.length>ss) ? line[ss].trim() : 0;
							if (notInt(pr.scanstations)) pr.scanstations = 0;
								
							pr.votingminutes = (line.length>vm) ? line[vm].trim() : -1; // -1 means use default minutes to vote
							if (notFloat(pr.votingminutes)) pr.votingminutes= -1;
							pr.checkinminutes = (line.length>cm) ? line[cm].trim() : -1; // -1 means use default minutes to check in
							if (notFloat(pr.checkinminutes)) pr.checkinminutes= -1;
							
							pr.registerminutes = (line.length>rm) ? line[rm].trim() : -1;
							if (notFloat(pr.registerminutes)) pr.registerminutes= -1;
							pr.scanminutes = (line.length>sm) ? line[sm].trim() : -1; 
							if (notFloat(pr.scanminutes)) pr.scanminutes= -1;
							
							// set arrival pattern to 0 to 9 (based on order in list, starting at 0)
							pr.arrivalpattern = (line.length>ap) ? line[ap].trim() : 0; 
							if (notInt(pr.arrivalpattern)) pr.arrivalpattern = 0;
							else pr.arrivalpattern = parseInt(pr.arrivalpattern);
							if (pr.arrivalpattern < 0 || pr.arrivalpattern > 9) pr.arrivalpattern = 0;
							
							pr.nocheckin = (line.length>nc) ? line[nc].trim() : -1; // -1 means use default
							if (notFloat(pr.nocheckin)) pr.nocheckin= -1;
							pr.earlyarrival = (line.length>ea) ? line[ea].trim() : -1; // -1 means use default
							if (notFloat(pr.earlyarrival)) pr.earlyarrival= -1;
							
							pr.maxwait=-99999;
							pr.avgwait=-99999;
							pr.walkoffs=-99999;
							
							// append to alldata.precinctdata
							alldata.precinctdata.push(pr);
						}
					}
					$('.data').css('display','inline');
					// uploaded data dictates same-day registration and scanner options, so disable user's ability to modify 
					$('#samedayregistration').prop('disabled',true); 
					$('#scannerlines').prop('disabled',true);
					showCounties();
					getPrecincts(); // calls drawChart
				}
			}
			///// end of anonymous function definition - function that parses the result of the next instruction. 
			r.readAsText(f); // THIS command executes, and on completion the above function parses the text.
		}
	}
	else
		alert("Please choose a file, or Cancel");
}

function downloadData()
{
	var c = alldata.columns; // Just a short local variable to reduce code for readability. See uploadData for column abbreviations.
	// alldata.columns = {c:c,p:p,t:t,r:r,vs:vs,cs:cs,rs:rs,ss:ss,vm:vm,cm:cm,rm:rm,sm:sm,ap:ap};
	// all above initialized to 99 and then set to actual column number if matching column identified in input
	var o; // o for output
	
	// Construct the header row
	if (c.c < 99) o = 'County,Precinct,Turnout'; // Precinct was required. Save the turnout value uploaded or used by default.
	else o = 'Precinct,Turnout';
	// Save the following only if included in original upload
	if (c.r < 99) o += ',Registration';
	if (c.ap < 99) o += ',Arrival Pattern';
	if (c.ea < 99) o += ',Early Arrival';
	if (c.nc < 99) o += ',Cannot Check In';
	if (c.cm < 99) o += ',Minutes to Check In';
	if (c.rm < 99) o += ',Minutes to Register';
	if (c.vm < 99) o += ',Minutes to Vote';
	if (c.sm < 99) o += ',Minutes to Scan';
	o += ',Check-in Stations';
	if (samedayreg) o += ',Registration Stations';
	o += ',Voting Stations';
	if (c.ss < 99) o += ',Scanners';
	// Values will be inserted into the following columns only if they differ from input (or they were not input)
	// The following loop checks whether above condition is true for these 4 optional but recommended columns 
	var dc=false; // difference in check-in times, voting times, registration times, scanning times
	var dv=false;
	var dr=false;
	var ds=false;
	for (var i=0; i<alldata.precinctdata.length; i++)
	{
		if (alldata.precinctdata[i].checkinminutes != alldata.precinctdata[i].checkminsused) dc = true;
		if (alldata.precinctdata[i].registerminutes != alldata.precinctdata[i].regminsused) dr = true;
		if (alldata.precinctdata[i].votingminutes != alldata.precinctdata[i].voteminsused) dv = true;
		if (alldata.precinctdata[i].scanminutes != alldata.precinctdata[i].scanminsused) ds = true;
	}
	// only include these columns if there were differences between input values and the values used in the simulation
	if (dc) o += ',Check Mins Used';
	if (samedayreg && dr) o += ',Reg Mins Used';
	if (dv) o += ',Vote Mins Used';
	if (c.ss < 99 && ds) o += ',Scan Mins Used';
	// The result columns:
	o += ',Max Wait,Avg Wait,Walkoffs\n';
	
	// Sort by ID so output is in same order as it was input
	alldata.precinctdata.sort(idCompare);
	unsorted=true;
	
	// Now append the data rows
	for (var i=0; i<alldata.precinctdata.length; i++)
	{
		var p = alldata.precinctdata[i]; // Use a short local variable for readability
		if (c.c < 99) o += p.county + ',' + p.precinct + ',' + p.turnout;
		else o += p.precinct + ',' + p.turnout;
		if (c.r < 99) o += ',' + p.registervoters;
		if (c.ap < 99) o += ',' + p.arrivalpattern;
		if (c.ea < 99) o += ',' + p.earlyarrival;
		if (c.nc < 99) o += ',' + p.nocheckin;
		if (c.cm < 99) o += ',' + ((parseFloat(p.checkinminutes)<0)?'':p.checkinminutes);
		if (c.rm < 99) o += ',' + ((parseFloat(p.registerminutes)<0)?'':p.registerminutes);
		if (c.vm < 99) o += ',' + ((parseFloat(p.votingminutes)<0)?'':p.votingminutes);
		if (c.sm < 99) o += ',' + ((parseFloat(p.scanminutes)<0)?'':p.scanminutes);
		o += ',' + p.checkinstations;
		if (samedayreg) o += ',' + p.regstations;
		o += ',' + p.votingstations;
		if (c.ss < 99) o += ',' + p.scanstations;
		// Only add data to these columns they differ from original input (for now this means "values were not input, using defaults")
		if (dc) o += ',' + ((p.checkinminutes != p.checkminsused)?p.checkminsused:'');
		if (samedayreg && dr) o += ',' + ((p.registerminutes != p.regminsused)?p.regminsused:'');
		if (dv) o += ',' + ((p.votingminutes != p.voteminsused)?p.voteminsused:'');
		if (c.ss < 99 && ds) o += ',' + ((p.scanminutes != p.scanminsused)?p.scanminsused:'');
		o += ',' + p.maxwait + ',' + p.avgwait + ',' + p.walkoffs +'\n';
	}
	
	// And output the CSV file
	var a = document.createElement('a');
	a.href = 'data:attachment/csv,' + escape(o);
	a.target = '_blank';
	a.download = 'capacity.csv';
	document.body.appendChild(a);
	a.click();
}



// This function launches the dialog box that prompts for a file to load. (Technically not an upload anymore, as all files
// are local now. But an early version worked differently and the function name wasn't changed.)
function showUploadDialog()
{
	// condition from 
	// http://www.htmlgoodies.com/beyond/javascript/read-text-files-using-the-javascript-filereader.html#fbid=bhDcNQYgkoV
	if (window.File && window.FileReader && window.FileList && window.Blob) 
	{
		document.getElementById("uploadDialog").style.display="block";
		document.getElementById("dialogWrapper").style.visibility="visible";
	} 
	else 
	{
		alert('A newer version of your browser is required for this feature.');
	}
	
}
function closeUploadDialog()
{
	document.getElementById("uploadDialog").style.display="none";
	document.getElementById("dialogWrapper").style.visibility="hidden";
}

////////////////////// A COUPLE OF UTILITY FUNCTIONS //////////////////////////////////////////////////

// notInt returns false if the value can be parsed as an integer and is only an integer.
// White space at the beginning and end is ignored, but any non-numeric character will cause
// this function to return true, even if "parseInt" would work. 
// For example, these will return true: "12abc" and "12.34" , even though parseInt would return
// the value of 12 for each.
function notInt(s)
{
	var v = parseInt(s);
	if (isNaN(v)) return true;
	if (v != s) return true;
	return false;
}

// notFloat returns false if the value can be parsed as a float and there are no extraneous 
// characters besides whitespace at the start or end. 
function notFloat(s)
{
	var v = parseFloat(s);
	if (isNaN(v)) return true;
	if (v != s) return true;
	return false;
}

// Parameter is jQuery string for a text box control. (e.g. '#checkinminutes')
// Check contents for a valid integer > 0. If not valid pop up an alert box 
// and put focus back on the control. Return true or false.
function validInt(c)
{
	var i = $(c).val();
	if (notInt(i) || i <= 0)
	{
		alert("Please enter an integer greater than zero.");
		$(c).focus();
		return false;
	}
	else return true;
}

// Same as above, but validates for floating point > 0			
function validFloat(c)
{
	var f = $(c).val();
	if (notFloat(f) || f <= 0)
	{
		alert("Please enter a number greater than zero.");
		$(c).focus();
		return false;
	}
	else return true;
}			
			
			
			